@@ -7,22 +7,22 @@
 #
 #
 
-__version__ = "0.2.1"
+__version__ = "0.2.2"
 # expose major functionality in dyntapy namespace such
 # that dyntapy.function() can be called directly
 from .assignments import DynamicAssignment, StaticAssignment
 from .demand_data import (
-    add_centroids_to_graph,
+    add_centroids,
     add_connectors,
     auto_configured_centroids,
     find_nearest_centroids,
     generate_random_od_graph,
     get_centroid_grid_coords,
+    _places_around_place,
 )
 from .graph_utils import get_shortest_paths, get_all_shortest_paths
 from .results import StaticResult, DynamicResult
-from .supply_data import places_around_place, relabel_graph, road_network_from_place
-from .toy_networks.get_networks import get_toy_network
+from .supply_data import relabel_graph, road_network_from_place, get_toy_network
 from .visualization import (
     show_demand,
     show_dynamic_network,
