@@ -14,49 +14,64 @@ import networkx as nx
 import numpy as np
 import osmnx as ox
 import pandas as pd
+from warnings import warn
+from numba import njit, prange
 from geojson import Feature, FeatureCollection, dumps
 from osmnx.distance import euclidean_dist_vec, great_circle_vec
-from scipy.sparse import lil_matrix
 from scipy.spatial import cKDTree
 from shapely.geometry import LineString, Point
 
 from dyntapy.settings import parameters
+from dyntapy.supply_data import default_buffer_dist
 from dyntapy.utilities import log
 
 
-def generate_random_od_graph(tot_ods, name, g, time, max_flow, seed=0):
+def generate_random_od_graph(tot_ods, name, g, max_flow=2000, seed=0):
     """
+    generates a random od-graph for a place
 
     Parameters
     ----------
-    tot_ods : total number of OD pairs to be generated
-    name : str, name of the city or region to geocode and sample from
-    g : nx.MultiDiGraph
-    time : time step to which this demand corresponds
-    max_flow : maximum demand per pair of points
-    seed : numpy random seed
+    tot_ods : int
+        total number of OD pairs to be generated
+    name : str
+        name of the city or region to geocode and sample from
+    g : networkx.DiGraph
+    max_flow : float, optional
+        maximum demand for any OD pair
+    seed : int, optional
+        random seed
 
     Returns
     -------
 
+    od_graph: networkx.DiGraph
+        graph with centroids as nodes with specified coordinates as 'x_coord' and
+        'y_coord'. For each OD pair with a non-zero demand there
+        is a link with a corresponding 'flow' element as read from the OD matrix.
     """
     json = generate_od_xy(tot_ods, name, max_flow, seed)
-    return parse_demand(json, g, time)
+    return parse_demand(json, g)
 
 
 def generate_od_xy(tot_ods, name: str, max_flow=2000, seed=0):
     """
+    generate random demand for a place in geojson format
 
     Parameters
     ----------
-    seed : numpy random seed
-    tot_ods :total number of OD pairs to be generated
-    name : str, name of the city or region to geocode and sample from
-    max_flow : maximum demand per pair of points
-
+    tot_ods : int
+        total number of OD pairs to be generated
+    name : str
+        name of the city or region to geocode and sample from
+    max_flow : float, optional
+        maximum demand for any OD pair
+    seed : int, optional
+        random seed
     Returns
     -------
-    geojson containing lineStrings and flows
+    geojson
+        containing LineStrings with a 'flow' attribute
     """
     # 4326 : WGS84
     # 3857 : web mercator
@@ -114,10 +129,7 @@ def _check_centroid_connectivity(g: nx.DiGraph):
     verifies if each centroid has at least one connector
     Parameters
     ----------
-    g : nx.Digraph
-
-    Returns
-    -------
+    g : networkx.Digraph
 
     """
     centroids = [u for u, data_dict in g.nodes(data=True) if "centroid" in data_dict]
@@ -139,24 +151,34 @@ def _check_centroid_connectivity(g: nx.DiGraph):
 
 def od_graph_from_matrix(od_matrix: np.ndarray, X, Y):
     """
-    creates od_graph(s) from od_matrix and centroid locations, if
+    creates od_graph from od_matrix and centroid locations
+
     Parameters
     ----------
-    od_matrix : float array, centroids x centroids
-    X : x_coord array of centroids
-    Y : y_coord array of centroids
+
+    od_matrix : numpy.ndarray
+        float, 2D
+    X : numpy.ndarray
+        float, 1D - lon of centroid locations
+    Y : numpy.ndarray
+        float, 1D - lat of centroid locations
 
     Returns
     -------
-    od_flow_graph: nx.DiGraph
+    od_graph: networkx.DiGraph
+        graph with centroids as nodes with specified coordinates as 'x_coord' and
+        'y_coord'. For each OD pair with a non-zero demand there
+        is a link with a corresponding 'flow' element as read from the OD matrix.
+
+
     """
     if od_matrix.shape != (len(X), len(X)):
         raise ValueError("dimensions of centroid locations and OD matrix incompatible")
-    g = nx.MultiDiGraph()
+    g = nx.DiGraph()
     nodes = [(u, {"x_coord": p[0], "y_coord": p[1]}) for u, p in enumerate(zip(X, Y))]
     g.add_nodes_from(nodes)
     edges = [
-        (u, v, 0, {"flow": od_matrix[u, v]})
+        (u, v, {"flow": od_matrix[u, v]})
         for u, v in np.argwhere(od_matrix > 0)
         if u != v
     ]
@@ -169,17 +191,26 @@ default_centroid_spacing = parameters.demand.default_centroid_spacing
 
 def get_centroid_grid_coords(name: str, spacing=default_centroid_spacing):
     """
+
     creates centroids on a grid that overlap with the polygon
-     that is associated with city or region specified
-    under 'name'
+    that is associated with city or region specified
+
+
     Parameters
     ----------
-    name : name of the city to be used as reference polygon
-    spacing : distance between two adjacent centroids on the grid
+    name : str,
+        name of the city to be used as reference polygon
+    spacing : float, optional
+        distance between two adjacent centroids on the grid
 
     Returns
     -------
-    X,Y arrays of centroid locations
+    X: numpy.ndarray
+        float, 1D
+        lon of centroid locations
+    Y: numpy.ndarray
+        float, 1D
+        lat of centroid locations
     """
     my_gdf = ox.geocode_to_gdf(name)
     range_ns_meters = great_circle_vec(
@@ -215,43 +246,58 @@ def get_centroid_grid_coords(name: str, spacing=default_centroid_spacing):
     return x, y
 
 
-def add_centroids_to_graph(g, X, Y, k=1, method="turn", euclidean=False, **kwargs):
+def add_centroids(g, X, Y, k=1, method="turn", euclidean=False, **kwargs):
     """
+
     Adds centroids to g.
-    Each centroids data dict contains 'x_coord','y_coord' and
-    'centroid' with x and y coords as given in X,Y
-    and 'centroid' a boolean set to True.
-    if method =='link':
-        k*2 connectors are added per centroid, one for each direction as defined below.
-    if method == 'turn':
-        k*2+2 connectors are added per centroid. There is another artificial
-        node between the centroid and the first
-        intersection node. All connector turns share the first starting link
-        from centroid to this artificial node.
-
-    any keyword arguments are passed as additional attributes into the graph.
-    They are assumed to be iterable and of
-    the same length as X
-
 
     Parameters
     ----------
-    euclidean : bool, toy networks use the euclidean coordinate system
-    method : ['turns' , 'links'] whether to add auto-configured link-connectors,
-     k*2 for each centroid
-    or auto-configured turn-connectors, k*2+2 for each centroid
-    Y : lat vector of centroids
-    X : lon vector of centroids
-    k : number of road network nodes to connect to per centroid.
-    g : nx.MultiDigraph containing only road network edges and nodes with 'x_coord'
-    and 'y_coord' attribute on each node
-    any keyword arguments are passed as additional attributes into the graph.
-    They are assumed to be iterable and of
-    the same length as X
+    g : networkx.Digraph
+        road network graph, containing only road network edges and nodes
+    X : numpy.ndarray
+        float, 1D - lon of centroids
+    Y : numpy.ndarray
+        float, 1D - lat of centroids
+    k : int
+        number of road network nodes to connect to per centroid.
+    method : {'turn' , 'link'}
+        whether to add link or turn connectors
+    euclidean : bool, optional
+        set to True for toy networks that use the euclidean coordinate system
+    **kwargs : iterable, optional
+        any keyword arguments are passed as additional attributes into the graph and
+        appear as attributes of the centroids.
+        They are assumed to be iterable and of
+        the same length as X.
 
     Returns
     -------
-    new nx.MultiDiGraph with centroids (and connectors)
+    networkx.DiGraph
+        new graph with centroids and connectors
+
+    Notes
+    -----
+    if method is 'link' k*2 connectors are added per centroid, one for each direction.
+    if method is 'turn'k*2+2 connectors are added per centroid. There is another
+    artificial node between the centroid and the first intersection node. All
+    connector turns share the first starting link from centroid to this artificial node.
+
+    The route choice algorithms for DTA within dyntapy rely on iterative computations on
+    the link-turn graph rather than the node-link graph.
+    Within the network one can take the current link and evaluate the options as the
+    set of all outgoing turns.
+    Evaluating the choice of the next turn to take from a centroid node without a
+    dummy starting link and turns is cumbersome because the structure differs.
+    We add these dummy turns to keep the algorithms simpler.
+
+
+
+    See Also
+    --------
+
+    dyntapy.demand_data.add_connectors
+
     """
     if len(kwargs) != 0:
         for key, val in zip(kwargs.keys(), kwargs.values()):
@@ -261,8 +307,8 @@ def add_centroids_to_graph(g, X, Y, k=1, method="turn", euclidean=False, **kwarg
                 raise TypeError(f"{key} is not iterable")
             if len(val) != len(X):
                 raise ValueError(f"{key} has the wrong dimension")
-    new_g = nx.MultiDiGraph()
-    new_g.graph = g.graph
+    new_g = nx.DiGraph()
+    new_g.graph = g.graph.copy()
     last_intersection_node = max(g.nodes)
     attributes = {"x_coord": X, "y_coord": Y, **kwargs}
     new_centroids = [
@@ -329,7 +375,7 @@ def add_centroids_to_graph(g, X, Y, k=1, method="turn", euclidean=False, **kwarg
             )
             add_connectors(
                 data["x_coord"],
-                data["y_coord"] + delta_y,
+                data["y_coord"],
                 j0 + j,
                 k,
                 g,
@@ -341,11 +387,47 @@ def add_centroids_to_graph(g, X, Y, k=1, method="turn", euclidean=False, **kwarg
 
 
 def auto_configured_centroids(
-    place: str,
-    buffer_dist_close: float,
+    place,
+    buffer_dist_close,
     buffer_dist_extended,
     inner_city_centroid_spacing=default_centroid_spacing,
 ):
+    """
+    generates centroids for the inner and extended study area from OpenStreetMap.
+    The inner area is filled with a grid.
+    The outer buffers are queried for settlements via OSMs 'place' tag.
+
+    Parameters
+    ----------
+    place: str
+        name of the city or region to buffer around.
+    buffer_dist_close: float
+        width of the inner buffer
+    buffer_dist_extended: float
+        width of the outer buffer
+    inner_city_centroid_spacing: float, optional
+        distance between two adjacent centroids on the grid
+
+    Returns
+    -------
+    X: numpy.ndarray
+        float, 1D
+        lon of centroid locations
+    Y: numpy.ndarray
+        float, 1D
+        lat of centroid locations
+    name: list of strings
+        name of the place as specified in OSM
+    place: list of strings
+        values for OSMs place tag
+
+
+    Notes
+    -----
+    The inner buffer is queried for places that are tagged as 'village', 'city',
+    or 'town'. The outer buffer is just querying for 'city' or 'town'.
+
+    """
     x_inner, y_inner = get_centroid_grid_coords(
         place, spacing=inner_city_centroid_spacing
     )
@@ -356,9 +438,11 @@ def auto_configured_centroids(
         place, {"place": ["city", "town"]}, buffer_dist=buffer_dist_extended
     )
     merged_gdf = pd.concat([gdf_close, gdf_extended])
-    # merged_gdf = gdf_close.append(gdf_extended)
     G = merged_gdf["geometry"].apply(lambda geom: geom.wkb)
     merged_gdf = merged_gdf.loc[G.drop_duplicates().index]
+    merged_gdf = merged_gdf[
+        merged_gdf.geometry.type == "Point"
+    ]  # considering only points
     names = merged_gdf["name"].tolist()
     x_ext, y_ext = (
         merged_gdf.geometry.apply(lambda x: x.x).to_numpy(dtype=np.float64),
@@ -378,20 +462,30 @@ def auto_configured_centroids(
 def add_connectors(x, y, u, k, g, new_g, euclidean):
     """
     adding connectors to new_g from starting node u with coordinates x and y to
-     nearest k intersection nodes in g.
+    nearest k intersection nodes in g.
+
     Parameters
     ----------
-    euclidean : bool, whether x and y are euclidean
-    x : lon
-    y : lat
-    u : connector from_node in new_g
-    k : number of (bidirectional) connectors to add
-    g : nx.MultiDiGraph containing all road network nodes
-    new_g : nx.MultiDiGraph containing at least u and all nodes of g
+    x : float
+        lon
+    y : float
+        lat
+    u : int
+        connector's from_node in new_g
+    k : int
+        number of (bidirectional) connectors to add
+    g : networkx.DiGraph
+        containing all road network nodes
+    new_g : networkx.DiGraph
+        containing at least u and all nodes of g
+    euclidean : bool
+        whether x and y are euclidean
+
+    Notes
+    -----
+
     default attributes of the connectors (speed, capacity, lanes)
-    can be changed in the settings.
-    Returns
-    -------
+    can be changed in dyntapy.settings.
 
     """
     tmp: nx.MultiDiGraph = g  # calculate distance to road network graph
@@ -401,9 +495,9 @@ def add_connectors(x, y, u, k, g, new_g, euclidean):
         # nearest nodes in consequent iterations if
         # multiple connectors are wanted
         if not euclidean:
-            v, length = get_nearest_node(tmp, (y, x), return_dist=True)
+            v, length = _get_nearest_node(tmp, (y, x), return_dist=True)
         else:
-            v, length = get_nearest_node(
+            v, length = _get_nearest_node(
                 tmp, (y, x), method="euclidean", return_dist=True
             )
         og_nodes.remove(v)
@@ -428,38 +522,152 @@ def add_connectors(x, y, u, k, g, new_g, euclidean):
         new_g.add_edge(v, u, **sink_data)
 
 
-def parse_demand(data: str, g: nx.DiGraph, time=0):
+def od_matrix_from_dataframes(
+    od_table: pd.DataFrame,
+    zoning: gpd.GeoDataFrame,
+    origin_column: str,
+    destination_column: str,
+    zone_column: str,
+    flow_column: str,
+    return_relabelling=False,
+):
     """
-    Maps travel demand to existing closest centroids in g.
-    The demand pattern is expressed as its own directed graph od_graph
-    returned with 'time', 'crs' and 'name' as metadata
-    in od_graph.graph : Dict.
-    The od_graph contains edges with a 'flow' entry that indicates the
-    movements from centroid to centroid.
-    The corresponding OD table can be retrieved through calling
-    nx.to_scipy_sparse_matrix(od_graph,weight='flow' )
+
+    extracts an OD matrix and X and Y coordinates as arrays from a pandas.DataFrame and
+    zoning provided as a geopandas.GeoDataFrame.
+
+    It is rather common to receive OD tables in the form of .csv files
+    and zoning in the form of .shp files. This function enables one to
+    extract the
+    full OD matrix after loading these files with pandas and geopandas,
+    respectively.
 
     Parameters
     ----------
-    time : time stamp for the demand data in hours (0<= time <=24)
-    data : geojson that contains lineStrings (WGS84) as features,
-     each line has an associated 'flow' stored in the properties dict
-    g : nx.MultiDigraph for the city under consideration with centroids assumed
-    to be labelled starting from 0, .. ,C-1
-    with C being the number of centroids.
 
-    There's no checking on whether the data and the nx.Digraph
-    correspond to the same geo-coded region.
+    od_table : pandas.DataFrame
+        each row should represent one entry in the OD matrix, with `origin_column`,
+        `destination_column` and `flow_column` as the relevant columns
+    zoning : gpd.GeoDataFrame
+        specifies geometries of the zoning, assumed to be in lon lat. The entries in
+        `zone_column` should correspond to the entries in `origin_column` and
+        `destination_column` in the `od_table`
+    origin_column: str
+    destination_column: str
+    zone_column: str
+    flow_column: str
+    return_relabelling: bool, optional
+        whether to return the mapping between the original zone labels in 'zone_column'
+        and the indexes in the returned OD matrix
+
     Returns
     -------
-    od_graph: nx.DiGraph
+
+    od_matrix: numpy.ndarray
+        float, 2D
+    X: np.ndarray
+        float, 1D, lon of centroids
+    Y: np.ndarray
+        float, 1D, lat of centroids
+    mapping: dict, optional
+
+    """
+
+    tot_zones = len(set(zoning[zone_column].to_list()))
+    assert len(set(zoning[zone_column].to_list())) == len(zoning[zone_column].to_list())
+    # no zone should have two conflicting geometries
+    mapping = dict(zip(zoning[zone_column], np.arange(tot_zones)))
+    od_table["_origin_idx"] = od_table[origin_column].apply(
+        lambda x: mapping.get(x, -1)
+    )
+    od_table["_destination_idx"] = od_table[destination_column].apply(
+        lambda x: mapping.get(x, -1)
+    )
+
+    od_matrix = np.zeros((tot_zones, tot_zones), dtype=np.float64)
+    try:
+        assert tot_zones > len(set(od_table[origin_column].to_list()))
+        assert tot_zones > len(set(od_table[destination_column].to_list()))
+    except AssertionError:
+        warn(
+            f"The values found in {zone_column=} do not fully contain all "
+            f"values found in "
+            f"{origin_column=} and/ or {destination_column=}. There are no "
+            f"geometries for some zones! The demand for those zones is omitted"
+            f"in the generated OD table."
+        )
+
+    flows = od_table[flow_column].to_numpy(dtype=np.float64, copy=True)
+    origins = od_table["_origin_idx"].to_numpy()
+    destinations = od_table["_destination_idx"].to_numpy()
+
+    @njit
+    def parse(_od_matrix, _origins, _destinations, _flows):
+        for idx in prange(len(_origins)):
+            i = _origins[idx]
+            j = _destinations[idx]
+            if i != j and i != -1 and j != -1:
+                # if either i or j do not have a geometry, we skip them.
+                # same for intra zonal traffic
+                _od_matrix[i][j] += _flows[idx]
+
+    # extracting centroid geometries from shapefile
+    zoning["centroid_x"] = zoning["geometry"].apply(lambda x: x.centroid.x)
+    zoning["centroid_y"] = zoning["geometry"].apply(lambda x: x.centroid.y)
+    parse(od_matrix, origins, destinations, flows)
+
+    if not return_relabelling:
+        return (
+            od_matrix,
+            zoning["centroid_x"].to_numpy(),
+            zoning["centroid_y"].to_numpy(),
+        )
+
+    else:
+        return (
+            od_matrix,
+            zoning["centroid_x"].to_numpy(),
+            zoning["centroid_y"].to_numpy(),
+            mapping,
+        )
+
+
+def parse_demand(data: str, g):
+    """
+    Maps travel demand to existing closest centroids in g.
+    The returned demand pattern is expressed as its own directed graph.
+
+    Parameters
+    ----------
+    data : geojson
+        that contains lineStrings (WGS84) as features,
+        each line has an associated 'flow' stored in the properties dictionary
+    g : networkx.Digraph
+
+
+    Returns
+    -------
+
+    od_graph: networkx.DiGraph
+        graph with centroids as nodes with specified coordinates as 'x_coord' and
+        'y_coord'. For each OD pair with a non-zero demand there
+        is a link with a corresponding 'flow' element as read from the OD matrix.
+
+    Notes
+    -----
+
+    There's no checking on whether the data and `g` correspond to the same geo-coded
+    region.
+
+    The corresponding OD table can be retrieved through calling
+
+    >>> networkx.to_scipy_sparse_matrix(od_graph,weight='flow')
 
     """
     od_graph = nx.MultiDiGraph()
-    od_graph.graph["time"] = time  # time in hours
     od_graph.graph["crs"] = "epsg:4326"
     name = g.graph["name"]
-    od_graph.graph["name"] = f"mobility flows in {name} at {time} s"
+    od_graph.graph["name"] = f"mobility flows in {name}"
     od_graph.add_nodes_from(
         [
             (u, data_dict)
@@ -505,15 +713,25 @@ def parse_demand(data: str, g: nx.DiGraph, time=0):
 
 def find_nearest_centroids(X, Y, centroid_graph: nx.DiGraph):
     """
+
+    finds the nearest centroids in the graph for a set of locations
+
     Parameters
     ----------
-    X : longitude of points epsg 4326
-    Y : latitude of points epsg 4326
-    centroid_graph : nx.DiGraph with existing centroids,
-    coordinates stored as 'x' and 'y' in epsg 4326
+    X : numpy.ndarray
+        longitude of points
+    Y : numpy.ndarray
+        latitude of points
+    centroid_graph : networkx.DiGraph
+        with existing centroids,
+        coordinates stored as 'x_coord' and 'y_coord' assumed to be lon and lat
 
     Returns
     -------
+    nearest_centroids, numpy.ndarray
+        int, 1D
+    distances, numpy.ndarray
+        in meter
 
     """
     if centroid_graph.number_of_nodes() == 0:
@@ -526,8 +744,6 @@ def find_nearest_centroids(X, Y, centroid_graph: nx.DiGraph):
         }
     )
     tree = cKDTree(data=centroids[["x", "y"]], compact_nodes=True, balanced_tree=True)
-    # ox.get_nearest_nodes()
-    # query the tree for nearest node to each origin
     points = np.array([X, Y]).T
     centroid_dists, centroid_idx = tree.query(points, k=1)
     snapped_centroids = centroids.iloc[centroid_idx].index
@@ -551,7 +767,7 @@ def _merge_gjsons(geojsons):
     return {"type": "FeatureCollection", "features": features}
 
 
-def get_nearest_node(G, point, method="haversine", return_dist=False):
+def _get_nearest_node(G, point, method="haversine", return_dist=False):
     """
     adapted from OSMNX
     Find the nearest node to a point.
@@ -622,96 +838,6 @@ def get_nearest_node(G, point, method="haversine", return_dist=False):
         return nearest_node
 
 
-def generate_od(
-    number_of_nodes,
-    origins_to_nodes_ratio,
-    origins_to_destinations_connection_ratio=0.15,
-    seed=0,
-):
-    """
-
-    Parameters
-    ----------
-    number_of_nodes : number of nodes (potential origins)
-    origins_to_nodes_ratio : float, indicates what
-    fraction of nodes are assumed to be origins
-    seed : seed for numpy random
-    origins_to_destinations_connection_ratio :
-
-    Returns
-    -------
-    od_matrix
-
-    """
-    rand_od = lil_matrix((number_of_nodes, number_of_nodes), dtype=np.uint32)
-    np.random.seed(seed)
-    number_of_origins = int(origins_to_nodes_ratio * number_of_nodes)
-    origins = np.random.choice(
-        np.arange(number_of_nodes), size=number_of_origins, replace=False
-    )
-    destinations = np.random.choice(
-        np.arange(number_of_nodes), size=number_of_origins, replace=False
-    )
-    for origin in origins:
-        # randomly sample how many and which destinations this origin is connected to
-        number_of_destinations = int(
-            np.random.gumbel(
-                loc=origins_to_destinations_connection_ratio,
-                scale=origins_to_destinations_connection_ratio / 2,
-            )
-            * len(destinations)
-        )
-        if number_of_destinations < 0:
-            continue
-        if number_of_destinations > len(destinations):
-            number_of_destinations = len(destinations)
-        destinations_by_origin = np.random.choice(
-            destinations, size=number_of_destinations, replace=False
-        )
-        rand_od.rows[origin] = list(destinations_by_origin)
-        rand_od.data[origin] = [
-            int(np.random.random() * 2000) for _ in destinations_by_origin
-        ]
-    return rand_od
-
-
-def generate_od_fixed(number_of_nodes, number_of_od_values, seed=0):
-    rand_od = lil_matrix((number_of_nodes, number_of_nodes), dtype=np.uint32)
-    np.random.seed(seed)
-    arr = np.arange(number_of_nodes * number_of_nodes)
-    vals = np.random.choice(arr, size=number_of_od_values, replace=False)
-    ids = [
-        np.where(arr.reshape((number_of_nodes, number_of_nodes)) == val) for val in vals
-    ]
-    for i, j in ids:
-        i, j = int(i), int(j)
-        if isinstance(rand_od.rows[i], list):
-            rand_od.rows[i].append(j)
-            rand_od.data[i].append(int(np.random.random() * 2000))
-        else:
-            rand_od.rows[i] = list(j)
-            rand_od.data[i] = list((int(np.random.random() * 2000)))
-    return rand_od
-
-
-def generate_random_bush(number_of_nodes, number_of_branches, seed=0):
-    rand_od = lil_matrix((number_of_nodes, number_of_nodes), dtype=np.uint32)
-    np.random.seed(seed)
-    origin = np.random.randint(0, number_of_nodes)
-    destinations = np.random.choice(
-        np.arange(0, number_of_nodes), number_of_branches, replace=False
-    )
-    for destination in destinations:
-        i, j = int(origin), int(destination)
-        if isinstance(rand_od.rows[i], list):
-            rand_od.rows[i].append(j)
-            rand_od.data[i].append(int(np.random.random() * 2000))
-        else:
-            rand_od.rows[i] = list(j)
-            rand_od.data[i] = list((int(np.random.random() * 2000)))
-    return rand_od
-
-
 def __count_iter_items(iterable):
     """
     Consume an iterable not reading it into memory; return the number of items.
@@ -724,3 +850,16 @@ def __count_iter_items(iterable):
 default_connector_speed = parameters.demand.default_connector_speed
 default_connector_capacity = parameters.demand.default_connector_capacity
 default_connector_lanes = parameters.demand.default_connector_lanes
+
+
+def _places_around_place(
+    place, buffer_dist=default_buffer_dist, tags=["city", "town", "village"]
+):
+    gdf = ox.geometries_from_place(place, {"place": tags}, buffer_dist=buffer_dist)
+    names = gdf["name"].tolist()
+    x, y = (
+        gdf.geometry.apply(lambda x: x.x).to_numpy(dtype=np.float),
+        gdf.geometry.apply(lambda x: x.y).to_numpy(dtype=np.float),
+    )
+    place_tags = gdf["place"].tolist()
+    return x, y, names, place_tags
