@@ -11,6 +11,7 @@
 #
 import datetime
 import os
+import warnings
 from inspect import signature
 from warnings import warn
 
@@ -29,22 +30,17 @@ from bokeh.tile_providers import Vendors, get_provider
 from pyproj import CRS
 from shapely.geometry import LineString
 
-from dyntapy.dta.time import SimulationTime
+from dyntapy.demand import SimulationTime
 from dyntapy.settings import parameters
-from dyntapy.supply_data import relabel_graph
 from dyntapy.utilities import __create_green_to_red_cm
-from dyntapy.results import StaticResult
 
 traffic_cm = __create_green_to_red_cm()
 
-default_max_links = parameters.visualization.max_links
 link_highlight_colors = parameters.visualization.link_highlight_colors
 node_highlight_color = parameters.visualization.node_highlight_color
 node_color = parameters.visualization.node_color
 centroid_color = parameters.visualization.centroid_color
-link_transparency = parameters.visualization.link_transparency
 node_size = parameters.visualization.node_size
-link_transparency = 0.8  # 0-1
 
 
 def _get_output_file(plot_name: str):
@@ -52,7 +48,7 @@ def _get_output_file(plot_name: str):
     return os.getcwd() + os.path.sep + f"{plot_name}_{dt_string}.html"
 
 
-def process_plot_arguments(g, title, notebook, toy_network, link_kwargs, node_kwargs):
+def _process_plot_arguments(g, title, notebook, euclidean, link_kwargs, node_kwargs):
     # process all arguments that are shared between dynamic and static
     for _, _, data in g.edges.data():
         if "x_coord" in data:
@@ -71,15 +67,15 @@ def process_plot_arguments(g, title, notebook, toy_network, link_kwargs, node_kw
             or not all(i == j - 1 for i, j in zip(linkids, linkids[1:]))
         ):
             needs_relabelling = True
-            # they have got to be
-            # starting at 0 & consecutively labelled integers
     except TypeError:
-        needs_relabelling = True
+        raise ValueError("Links in g were not labelled, use relabel_graph function")
     if needs_relabelling:
         raise ValueError(
             "Links in g were not labelled correctly, use relabel_graph function"
         )
-    if not toy_network:
+        # they have got to be
+        # starting at 0 & consecutively labelled integers
+    if not euclidean:
         plot = figure(
             plot_height=parameters.visualization.plot_size,
             plot_width=parameters.visualization.plot_size,
@@ -90,9 +86,11 @@ def process_plot_arguments(g, title, notebook, toy_network, link_kwargs, node_kw
         )
         tile_provider = get_provider(Vendors.CARTODBPOSITRON_RETINA)
         plot.add_tile(tile_provider)
+        tmp = nx.MultiDiGraph(g)
         tmp = ox.project_graph(
-            g, CRS.from_user_input(3857)
+            tmp, CRS.from_user_input(3857)
         )  # from lan lot to web mercator
+
     else:
         plot = figure(
             plot_height=parameters.visualization.plot_size,
@@ -135,27 +133,96 @@ def process_plot_arguments(g, title, notebook, toy_network, link_kwargs, node_kw
 
 
 def show_network(
-    g: nx.MultiDiGraph,
-    result: StaticResult = None,
+    g,
     flows=None,
     link_kwargs=dict(),
     node_kwargs=dict(),
     highlight_links=np.array([]),
     highlight_nodes=np.array([]),
+    euclidean=False,
     toy_network=False,
     title=None,
     notebook=False,
     show_nodes=True,
 ):
-    # adding different coordinate attribute names to use osmnx functions
-    plot, tmp = process_plot_arguments(
-        g, title, notebook, toy_network, link_kwargs, node_kwargs
+    """
+    Visualizing a network with static attributes in a .html.
+
+    Parameters
+    ----------
+    g: networkx.DiGraph
+    flows: numpy.ndarray, optional
+        float, 1D - flows to be visualized
+    link_kwargs: dict, optional
+        additional link information to be displayed
+        key: str, value: numpy.ndarray - additional link information to be
+        displayed
+    node_kwargs: dict, optional
+        key: str, value: numpy.ndarray - additional node information to be
+        displayed
+    highlight_links: numpy.ndarray or list, optional
+        int, 1D or 2D - links to highlight
+    highlight_nodes: numpy.ndarray or list, optional
+        int, 1D or 2D - links to highlight
+    euclidean: bool, optional
+        set to True if coordinates in graph are euclidean.
+    toy_network: bool, optional
+        deprecated, use euclidean instead.
+    title: str, optional
+    notebook: bool, optional
+        set to True if the plot should be rendered in a notebook.
+    show_nodes: bool, optional
+        whether to render nodes
+
+    Examples
+    --------
+
+    >>> show_network(g, highlight_links=[[2,4],[3,6]])
+
+    will plot the network and highlight links [2,4] in neon pink,
+    and [3, 6] in cyan. The order of highlight colors is neon pink, cyan, lime green,
+    light blue, orange, gray
+
+    Node highlighting works analogously.
+
+    >>> foo = np.arange(g.number_of_edges())
+    >>> bar = np.arange(g.number_of_edges())
+    >>> show_network(g, link_kwargs={'foo': foo, 'bar':bar})
+
+    will generate a plot where respective values for `foo` and `bar` can be inspected by
+    hovering over the link.
+    Note that the string attribute names cannot contain spaces and that the arrays
+    must have the correct dimension.
+
+    node_kwargs can be passed on analogously.
+
+
+    Notes
+    -----
+
+    highlight colors can be altered in the settings and have been chosen to still offer
+    visibility in a graph with loaded traffic in a green-to-red color map.
+
+    See Also
+    -------------
+
+    dyntapy.results.StaticResult
+
+    """
+
+    if toy_network:
+        euclidean = toy_network
+        warnings.warn(
+            "use of toy_network arg is deprecated, use euclidean instead",
+            DeprecationWarning,
+        )
+
+    plot, tmp = _process_plot_arguments(
+        g, title, notebook, euclidean, link_kwargs, node_kwargs
     )
     show_flows = True
     if flows is not None:
         pass
-    elif result is not None:
-        flows = result.flows
     else:
         flows = np.zeros(g.number_of_edges())
         show_flows = False
@@ -187,7 +254,7 @@ def show_network(
             fill_color="color",
             line_color="black",
             line_alpha=0.8,
-            fill_alpha=link_transparency,
+            fill_alpha=parameters.visualization.link_transparency,
         ),
     )
     edge_tooltips = [
@@ -234,7 +301,23 @@ def show_network(
 
 
 def show_link_od_flows(g: nx.DiGraph, od_flows, **kwargs):
-    # same keyword arguments as show_network
+    """
+    Visualizing a network with origin destination flows for each link in a .html.
+
+    Parameters
+    ----------
+    g: networkx.DiGraph
+    od_flows: list
+        origin destination flows for each link
+    kwargs: any
+        all the arguments of `show_network` are valid, except for `flows`
+
+    See Also
+    --------
+
+    dyntapy.visualization.show_network
+
+    """
     formatted_od_flows = []
     tot_flows = np.zeros(len(od_flows), dtype=np.float64)
     for link in range(len(od_flows)):
@@ -253,12 +336,13 @@ def show_link_od_flows(g: nx.DiGraph, od_flows, **kwargs):
 
 
 def show_dynamic_network(
-    g: nx.DiGraph,
-    time: SimulationTime,
+    g,
+    time,
     flows=None,
     link_kwargs=dict(),
     node_kwargs=dict(),
     toy_network=False,
+    euclidean=False,
     highlight_nodes=np.array([]),
     highlight_links=np.array([]),
     title=None,
@@ -266,21 +350,75 @@ def show_dynamic_network(
     show_nodes=True,
 ):
     """
+    Visualizing a network with dynamic attributes in a .html.
 
     Parameters
     ----------
-    notebook
-    flows
-    g : nx.Digraph
-    title : str, plot title
-    plot_size : height and width measurement in pixel
+    g: networkx.DiGraph
+    time: dyntapy.demand.SimulationTime
+    flows: numpy.ndarray, optional
+        float, 2D - time-dependent flows to be visualized
+    link_kwargs: dict, optional
+        additional time-dependent link information to be displayed
+        key: str, value: np.ndarray - additional time-dependent link information to
+        be displayed
+    node_kwargs: dict, optional
+        key: str, value: np.ndarray - additional time-dependent node information to
+        be displayed
+    highlight_links: numpy.ndarray, optional
+        int, 1D or 2D - links to highlight
+    highlight_nodes: numpy.ndarray, optional
+        int, 1D or 2D - links to highlight
+    euclidean: bool, optional
+        set to True if coordinates in graph are euclidean.
+    toy_network: bool, optional
+        deprecated, use euclidean instead.
+    title: str, optional
+    notebook: bool, optional
+        set to True if the plot should be rendered in a notebook.
+    show_nodes: bool, optional
+        whether to render nodes
 
-    Returns
-    -------
+    Examples
+    --------
+    highlighting works just as in `show_network` and is not time dependent.
+
+    >>> foo = np.arange((g.number_of_edges(), time.tot_time_steps))
+    >>> bar = np.arange((g.number_of_edges(), time.tot_time_steps))
+    >>> show_dynamic_network(g, link_kwargs={'foo': foo, 'bar':bar})
+
+    Will generate a plot where respective values for `foo` and `bar` can be inspected by
+    hovering over the link.
+    The values are updated as the time slider is moved.
+
+    Note that the string attribute names cannot contain spaces and that the arrays
+    must have the correct dimension.
+
+    node_kwargs can be passed on analogously.
+
+
+    Notes
+    -----
+
+    highlight colors can be altered in the settings and have been chosen to still offer
+    visibility in a graph with loaded traffic in a green-to-red color map.
+
+    See Also
+    -------------
+
+    dyntapy.results.DynamicResult
 
     """
-    plot, tmp = process_plot_arguments(
-        g, title, notebook, toy_network, link_kwargs, node_kwargs
+
+    if toy_network:
+        euclidean = toy_network
+        warnings.warn(
+            "use of toy_network arg is deprecated, use euclidean instead",
+            DeprecationWarning,
+        )
+
+    plot, tmp = _process_plot_arguments(
+        g, title, notebook, euclidean, link_kwargs, node_kwargs
     )
     if flows is None:
         if "flows" not in list(link_kwargs.keys()):
@@ -367,7 +505,7 @@ def show_dynamic_network(
         glyph=Patches(
             xs="x",
             ys="y",
-            fill_alpha=link_transparency,
+            fill_alpha=parameters.visualization.link_transparency,
             fill_color="color",
             line_color="black",
             line_alpha=0.4,
@@ -493,7 +631,44 @@ def get_max_edge_width(g, scaling, plot_size):
     return max_width_bokeh, max_width_coords
 
 
-def show_demand(g, title=None, notebook=False, toy_network=False):
+def show_demand(g, title=None, notebook=False, euclidean=False, toy_network=False):
+    """
+
+    visualize demand on a map
+
+    Parameters
+    ----------
+    g: networkx.DiGraph
+    title: str
+    notebook: bool, optional
+        set to True if the plot should be rendered in a notebook.
+    euclidean: bool, optional
+        set to True, if 'x_coord' and 'y_coord' in g are euclidean.
+    toy_network: bool, optional
+        deprecated, use euclidean instead
+
+    Examples
+    --------
+
+    Given an `od_matrix` and coordinates in longitude `X` and latitude `Y` it is
+    straightforward to visualize the travel pattern.
+
+    >>> od_graph_from_matrix(od_matrix, X, Y)
+    >>> show_demand(g)
+
+    See Also
+    --------
+
+    dyntapy.demand_data.od_graph_from_matrix
+
+    """
+    if toy_network:
+        euclidean = toy_network
+        warnings.warn(
+            "use of toy_network arg is deprecated, use euclidean instead",
+            DeprecationWarning,
+        )
+
     for _, _, data in g.edges.data():
         if "x_coord" in data:
             data["x"] = data["x_coord"]
@@ -509,9 +684,10 @@ def show_demand(g, title=None, notebook=False, toy_network=False):
         output_notebook(hide_banner=True)
     else:
         output_file(filename=_get_output_file(title))
-    if not toy_network:
+    if not euclidean:
         g.graph["crs"] = "epsg:4326"
-        tmp = ox.project_graph(g, CRS.from_user_input(3857))
+        tmp = nx.MultiDiGraph(g)
+        tmp = ox.project_graph(tmp, CRS.from_user_input(3857))
         plot = figure(
             plot_height=parameters.visualization.plot_size,
             plot_width=parameters.visualization.plot_size,
