@@ -14,14 +14,13 @@ import numpy as np
 one_up = pathlib.Path(__file__).parents[1]
 sys.path.append(one_up.as_posix())
 
-from dyntapy.toy_networks.get_networks import get_toy_network
 from dyntapy.demand import DynamicDemand
 from dyntapy.demand_data import od_graph_from_matrix
-from dyntapy.dta.time import SimulationTime
 from dyntapy.assignments import DynamicAssignment
 from dyntapy.dta.orca_nodel_model import orca_node_model
-from dyntapy import get_shortest_paths, get_all_shortest_paths
-from dyntapy.demand_data import generate_od_xy, add_centroids_to_graph, \
+from dyntapy import get_shortest_paths, get_all_shortest_paths, get_toy_network
+from dyntapy.demand import SimulationTime
+from dyntapy.demand_data import generate_od_xy, add_centroids, \
     auto_configured_centroids, parse_demand
 from dyntapy import StaticAssignment
 from dyntapy import show_demand, show_network, show_dynamic_network, \
@@ -30,7 +29,7 @@ from dyntapy.results import get_od_flows, get_selected_link_analysis
 from dyntapy.supply_data import road_network_from_place, relabel_graph
 from dyntapy.results import StaticResult
 
-city = 'Zinnowitz'
+_city = 'Leuven'
 graph = None
 result: [StaticResult] = None
 demand = None
@@ -39,10 +38,10 @@ assignment = None
 HERE = pathlib.Path(__file__).parent
 one_up = pathlib.Path(__file__).parents[1]
 sys.path.append(one_up.as_posix())
-file_path_network = HERE.as_posix() + os.path.sep + city.lower() + '_road_network'
+file_path_network = HERE.as_posix() + os.path.sep + _city.lower() + '_road_network'
 
 
-def test_get_graph(city=city, k=1, connector_type='link'):
+def test_get_graph(city=_city, k=4, connector_type='link'):
     """
 
     Parameters
@@ -61,10 +60,10 @@ def test_get_graph(city=city, k=1, connector_type='link'):
     x, y, names, place_tags = auto_configured_centroids(city, buffer_dist_close=5000,
                                                         buffer_dist_extended=10000)
     print('centroids found')
-    g = add_centroids_to_graph(g, x, y, k=k, method=connector_type,
-                               name=names, place=place_tags)
+    g = add_centroids(g, x, y, k=k, method=connector_type,
+                      name=names, place=place_tags)
     print('centroids added to graph')
-    g = relabel_graph(g)
+    g, inverse = relabel_graph(g, return_inverse=True)
     with open(file_path_network, 'wb') as network_file:
         dump(g, network_file)
         print(f'network saved at f{file_path_network}')
@@ -75,8 +74,8 @@ def test_get_graph(city=city, k=1, connector_type='link'):
 
 test_get_graph()
 seed = 1
-json_demand = generate_od_xy(30, city, seed=seed, max_flow=200)
-od_graph = parse_demand(json_demand, graph, 0)
+json_demand = generate_od_xy(40, _city, seed=seed, max_flow=900)
+od_graph = parse_demand(json_demand, graph)
 
 
 def test_shortest_path():
@@ -98,7 +97,7 @@ def test_dial_b():
     demand = assignment.internal_demand
     method = 'dial_b'
     result = assignment.run(method)
-    show_network(graph, result)
+    show_network(graph, flows=result.flows)
     print(f'DUE {method=} ran successfully')
 
 
@@ -107,7 +106,7 @@ def test_msa():
     method = 'msa'
     res = loc_assignment.run(method)
     print(f'DUE {method=} ran successfully')
-    show_network(graph, res)
+    show_network(graph, res.flows)
 
 
 def test_node_model():
@@ -159,7 +158,7 @@ def test_node_model():
 def test_selected_link_analysis():
     link = 622
     od_flows = get_od_flows(assignment, result)
-    show_link_od_flows(graph, od_flows, result=result)
+    show_link_od_flows(graph, od_flows)
     sla_od_flows = get_selected_link_analysis(assignment, od_flows, link)
     show_link_od_flows(graph, sla_od_flows, highlight_links=[link])
 
@@ -168,28 +167,44 @@ def test_sun():
     loc_assignment = StaticAssignment(graph, od_graph)
     method = 'sun'
     res = loc_assignment.run(method)
-    show_network(graph, res)
+    show_network(graph, res.flows)
+
+
+def test_get_toy_networks():
+    for name in [
+        "cascetta",
+        "simple_bottleneck",
+        "simple_merge",
+        "simple_diverge",
+        "chicagosketch",
+        "chicagoregional",
+        "siouxfalls",
+        "birmingham"]:
+        g = get_toy_network(name)
+        g = relabel_graph(g)
+        show_network(g, euclidean=True)
+        print(f'{name=}')
 
 
 def test_dta():
     g = get_toy_network('cascetta')
     centroid_x = np.array([1, 7, 4])
     centroid_y = np.array([1, 1, 3.5])
-    g = add_centroids_to_graph(g, centroid_x, centroid_y, euclidean=True)
+    g = add_centroids(g, centroid_x, centroid_y, euclidean=True)
     # also adds connectors automatically
     g = relabel_graph(g)  # adding link and node ids, connectors and centroids
     # are the first elements
-    show_network(g, toy_network=True)
+    show_network(g, euclidean=True)
     od_matrix = np.zeros(9).reshape((3, 3))
     od_matrix[0, 1] = 500
     od_matrix[2, 1] = 500
     od_graph = od_graph_from_matrix(od_matrix, centroid_x, centroid_y)
-    show_demand(od_graph, toy_network=True)
+    show_demand(od_graph, euclidean=True)
     dynamic_demand = DynamicDemand([od_graph], insertion_times=[0])
     # convert everything to internal representations and parse
     simulation_time = SimulationTime(np.float32(0.0), np.float32(2.0), step_size=0.25)
     assignment = DynamicAssignment(g, dynamic_demand, simulation_time)
     result = assignment.run()
-    show_dynamic_network(g, simulation_time, flows=result.flows, toy_network=True,
+    show_dynamic_network(g, simulation_time, flows=result.flows, euclidean=True,
                          link_kwargs={'costs': result.link_costs},
                          )
